<?php
/* Free Shipping PRO v1.3.0 */
class ControllerExtensionShippingFreePro extends Controller {
	
	public function geolocation(){
		$json = array();
		if ($this->request->server['REQUEST_METHOD'] == 'POST' && isset($this->request->post['ip'])) {	
			
			if (isset($this->request->post['debug']) && !empty($this->request->post['debug'])) {
				$this->session->data['debugIP'] = $ip = $this->request->post['ip'];
			} elseif (!empty($this->session->data['debugIP'])) {
				$ip = $this->session->data['debugIP'];
			} else {
				$ip = $this->request->post['ip'];
				unset($this->session->data['debugIP']);
			}
						
			$service = $this->config->get('shipping_free_pro_geolocation_service');
			$json = $this->getLocation($ip, $service);
		}
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));		
	}
	
	protected function getLocation($ip, $service) {
		
		switch ($service) {
			case 'freegeoip': 
				$response = $this->getFreegeoip($ip);
				break;
			case 'geoplugin':
				$response = $this->getGeoplugin($ip);
				break;
			case 'ipapicom':
				$response = $this->getIpApiCom($ip);
				break;
			case 'telize':
				/* DEPRECATED, service no longer active */
				$response = $this->getTelize($ip);
				break;
			
		}
		
		if (!empty($response)) {
			$response['DBGeoData'] = $this->getDBGeoData($response['reformat']);
			return $response;	
		} else {
			return 'Service Unavailable';
		}
	}
	
	protected function getFreegeoip($ip) {
	
		$response = $this->getContents($ip, 'http://freegeoip.net/json/');
		if (empty($response)) {
			$this->session->data['geoip_success'] = 0;
			if ($this->config->get('shipping_free_pro_service_fallback') == 'on') { 
				$response = $this->getGeoplugin($ip);
			}
		} else {
			$response = $this->reformatFreeGeoIpResponse($response);
			$this->session->data['geoip_success'] = 1;
		}
		return$response;
		
	}
	
	protected function getGeoplugin($ip) {
	
		$response = $this->getContents($ip, 'http://www.geoplugin.net/json.gp?ip=');
		
		if (empty($response)) {
			$this->session->data['geoip_success'] = 0;
			if ($this->config->get('shipping_free_pro_service_fallback') == 'on') { 
				$response = $this->getIpApiCom($ip);
			}
		} else {
			$response = $this->reformatGeopluginResponse($response);
			$this->session->data['geoip_success'] = 1;
		}
		
		return $response;
		
	}
	
	protected function getIpApiCom($ip) {
	
		$response = $this->getContents($ip, 'http://ip-api.com/json/');
		
		if (empty($response)) {
			$this->session->data['geoip_success'] = 0;
			if ($this->config->get('shipping_free_pro_service_fallback') == 'on') { 
				$response = $this->getFreegeoip($ip);
			}
		} else {
			$response = $this->reformatIpApiComResponse($response);
			$this->session->data['geoip_success'] = 1;
		}
		
		return $response;
		
	}
	
	protected function getTelize($ip) {
		/* DEPRECATED, service no longer active */
		$response = $this->getContents($ip, 'http://www.telize.com/geoip/');
		
		if (empty($response)) {
			$this->session->data['geoip_success'] = 0;
			if ($this->config->get('shipping_free_pro_service_fallback') == 'on') { 
				$response = $this->getFreegeoip($ip);
			}
		} else {
			$response = $this->reformatTelizeResponse($response);
			$this->session->data['geoip_success'] = 1;
		}
		
		return $response;
		
	}
	
	protected function reformatFreeGeoIpResponse($d) {
	
		$data = json_decode($d, true);
		
		$response = array(
			'ip' 			=> $data['ip'],
			'country_code' 	=> $data['country_code'],
			'country_name' 	=> $data['country_name'],
			'city'			=> $data['city'],
			'region_name'	=> $data['region_name'],
		);
	
		return array('reformat' => $response, 'raw' => $data);
	}
	
	protected function reformatGeopluginResponse($d) {
	
		$geoplugin_data = json_decode($d, true);
		
		$response = array(
		
			'ip' 			=> $geoplugin_data['geoplugin_request'],
			'country_code' 	=> $geoplugin_data['geoplugin_countryCode'],
			'country_name' 	=> $geoplugin_data['geoplugin_countryName'],
			'city'			=> $geoplugin_data['geoplugin_city'],
			'region_name'	=> $geoplugin_data['geoplugin_regionName'],
		);
	
		return array('reformat' => $response, 'raw' => $geoplugin_data);
	}
	
	protected function reformatIpApiComResponse($d) {
	
		$data = json_decode($d, true);
		
		$response = array(
			'ip' 			=> $data['query'],
			'country_code' 	=> $data['countryCode'],
			'country_name' 	=> $data['country'],
			'city'			=> $data['city'],
			'region_name'	=> $data['regionName'],
		);
	
		return array('reformat' => $response, 'raw' => $data);
	}
	
	
	protected function reformatTelizeResponse($d) {
	
		$data = json_decode($d, true);
		
		$response = array(
		
			'ip' 			=> $data['ip'],
			'country_code' 	=> $data['country_code'],
			'country_name' 	=> $data['country'],
			'city'			=> empty($data['city']) ? '' : $data['city'],
			'region_name'	=> empty($data['region']) ? '' : $data['region']
		);
	
		return array('reformat' => $response, 'raw' => $data);
	}
	
	protected function getContents($ip, $url) {
		
		$url .= $ip;
		$response = file_get_contents("$url");
		
		return $response;
		
	}
	
	protected function getDBGeoData($geo_data) {
		
		$return = array();
		$status = $this->config->get('shipping_free_pro_status');
		
		$fsp_debug = $this->config->get('shipping_free_pro_debug_status');
		
		if ($fsp_debug) {
			/* If debug mode active, reset session variables */
			$this->session->data['country_data'] = array();
			$this->session->data['fsp'] = array();
			$this->session->data['fsp_match'] = 0;
			$country = array();
			unset($this->session->data['geo_data']);
		}
		
		if ($status) {
			$this->session->data['geo_data'] = $geo_data;
			
			$return['geo_data'] = $geo_data;
			
			if ($this->customer->isLogged() == false) {
				$customer_group_id = 0;
			} else {
				$customer_group_id = $this->customer->getGroupId();
			}
					
			$store_id = $this->config->get('config_store_id');
			
			if (empty($country)) {
				$country = $this->getCountryData($geo_data['country_code']);
				$this->session->data['country_data'] = $country;
			}
					
			if (empty($this->session->data['fsp_match']) && isset($country)) {
				$fsp = $this->config->get('shipping_free_pro_data');
				if (!empty($country['country_id'])) {
					foreach ($fsp as $key) {
						if ($key['country_id'] == $country['country_id'] && in_array($customer_group_id, $key['customer_group_id']) && in_array($store_id, $key['store_id'])) {
								$this->session->data['fsp_match'] = 1;
								$return['fsp'] = $this->session->data['fsp'] = $key;
							break;
						} else {
							$this->session->data['fsp_match'] = 0;
						}
					}
				
					if (!empty($this->session->data['fsp'])) {
						foreach ($fsp as $key) {
							if ($this->session->data['fsp']['country_id'] == $key['country_id'] && $this->session->data['fsp']['zone_id'] == 0 && $this->session->data['fsp']['total'] > $key['total']) { 
								$return['fsp'] = $this->session->data['fsp'] = $key;
								if ($key['country_id'] == $country['country_id'] && in_array($customer_group_id, $key['customer_group_id']) && in_array($store_id, $key['store_id'])) {
									$this->session->data['fsp_match'] = 1;
								} else {
									$this->session->data['fsp_match'] = 0;
								}
								break;
							}
						}
					}
				}
			}
			return $return;
		} else {
			return false;
		}
	}
	
	protected function getCountryData($data) {
		$query = $this->db->query("SELECT country_id, name, iso_code_2, iso_code_3 FROM `" . DB_PREFIX . "country` WHERE `iso_code_2` = '" . $this->db->escape($data) . "' AND `status` = '1'");
		return $query->row;
	}
	
	protected function getZoneId($data) {
		$query = $this->db->query("SELECT zone_id FROM `" . DB_PREFIX . "zone` WHERE name = '" . $data . "' AND status = '1'");
		return $query->row;
	}

}